package attribute_test

import (
	"gitlab.codemonkeysoftware.net/b/hatmill/attribute"
	"testing"
)

func expectEqualStrings(t *testing.T, actual, expected string) {
	if actual != expected {
		t.Fatalf("expected %q, got %q", expected, actual)
	}
}

func TestSpaceList(t *testing.T) {
	t.Run("empty", func(t *testing.T) {
		expectEqualStrings(t, attribute.SpaceList{}.String(), "")
	})

	t.Run("nonempty", func(t *testing.T) {
		expected := "alpha bravo charlie"
		actual := attribute.SpaceList{"alpha", "bravo", "charlie"}.String()
		expectEqualStrings(t, actual, expected)
	})
}

func TestCommaList(t *testing.T) {
	t.Run("empty", func(t *testing.T) {
		expectEqualStrings(t, attribute.CommaList{}.String(), "")
	})

	t.Run("nonempty", func(t *testing.T) {
		expected := "alpha,bravo,charlie"
		actual := attribute.CommaList{"alpha", "bravo", "charlie"}.String()
		expectEqualStrings(t, actual, expected)
	})
}

func TestFloat(t *testing.T) {
	for _, testcase := range []struct {
		name string
		f    attribute.Float
		s    string
	}{
		{name: "zero", f: 0, s: "0"},
		{name: "integral", f: 45, s: "45"},
		{name: "positive fractional, small positive exponent", f: 12.55, s: "12.55"},
		{name: "positive fractional, large positive exponent", f: 1.55E+12, s: "1.55E+12"},
		{name: "positive fractional, large negative exponent", f: 1.55E-12, s: "1.55E-12"},
		{name: "negative fractional, small positive exponent", f: -12.55, s: "-12.55"},
		{name: "negative fractional, large positive exponent", f: -1.55E+12, s: "-1.55E+12"},
		{name: "negative fractional, large negative exponent", f: -1.55E-12, s: "-1.55E-12"},
	} {
		t.Run(testcase.name, func(t *testing.T) {
			expectEqualStrings(t, testcase.f.String(), testcase.s)
		})
	}
}

func TestInt(t *testing.T) {
	t.Run("zero", func(t *testing.T) {
		expectEqualStrings(t, attribute.Int(0).String(), "0")
	})
	t.Run("positive", func(t *testing.T) {
		expectEqualStrings(t, attribute.Int(45).String(), "45")
	})
	t.Run("negative", func(t *testing.T) {
		expectEqualStrings(t, attribute.Int(-45).String(), "-45")
	})
}

func TestExplicitBool(t *testing.T) {
	t.Run("true", func(t *testing.T) {
		expectEqualStrings(t, attribute.Bool(true).String(), "true")
	})
	t.Run("false", func(t *testing.T) {
		expectEqualStrings(t, attribute.Bool(false).String(), "false")
	})
}

func TestString(t *testing.T) {
	const s = "abcdefg"
	expectEqualStrings(t, attribute.String(s).String(), s)
}
